Enum russell_sparse::Ordering

source ·
pub enum Ordering {
    Amd,
    Amf,
    Auto,
    Best,
    Cholmod,
    Colamd,
    Metis,
    No,
    Pord,
    Qamd,
    Scotch,
}
Ordering option

Variants



Amd

Ordering using the approximate minimum degree



Amf

Ordering using the approximate minimum fill-in (MUMPS-only, otherwise Auto)



Auto

Automatic ordering method selection



Best

Try three methods and take the best (UMFPACK-only, otherwise Auto)



Cholmod

Use Amd for symmetric, Colamd for unsymmetric, or Metis (UMFPACK-only, otherwise Auto)



Colamd

Use the column approximate minimum degree ordering algorithm (KLU-only, otherwise Auto)



Metis

Ordering by Karpis & Kumar from the University of Minnesota



No

The matrix is factorized as-is (UMFPACK-only, otherwise Auto)



Pord

Ordering by Schulze from the University of Paderborn (MUMPS-only, otherwise Auto)



Qamd

Ordering using the automatic quasi-dense row detection (MUMPS-only, otherwise Auto)



Scotch

Ordering using the Scotch package (MUMPS-only, otherwise Auto)

Implementations

source

impl Ordering

source

pub fn from(ordering: &str) -> Self

Returns the Ordering by name (default is Auto)