pub enum Pivoting {
Auto,
None,
GlobalCol,
GlobalRow,
Diagonal,
LocalBlock,
}Expand description
Specifies the pivoting strategy (cuDSS only)
See: https://docs.nvidia.com/cuda/cudss/types.html#cudsspivottype-t
Variants
Auto
Automatic pivoting strategy (default)
Automatically selects the appropriate pivot type based on the reordering algorithm and matrix type:
- For Ordering::Auto and Ordering::Amd:
- Symmetric/Hermitian indefinite matrices: Uses Pivoting::Diagonal
- General matrices: Uses Pivoting::LocalBlock
- For Ordering::BtfColamd and Ordering::Colamd: Uses Pivoting::GlobalCol
CUDSS_PIVOT_AUTO
None
GlobalCol
GlobalRow
Diagonal
Diagonal pivoting
Used with Ordering::Auto and Ordering::Amd (MetisND and AMD).
For symmetric/Hermitian indefinite matrices, searches for pivot elements only within the diagonal of the supernode.
CUDSS_PIVOT_DIAGONAL
LocalBlock
Complete block pivoting
Use with Ordering::Auto and Ordering::Amd.
For general matrices, searches for pivot elements within the entire diagonal block of the supernode.
CUDSS_PIVOT_LOCAL_BLOCK
Implementations