Type Alias russell_sparse::prelude::ComplexCooMatrix

source ·
pub type ComplexCooMatrix = NumCooMatrix<Complex64>;
Defines an alias to NumCooMatrix with Complex64

Aliased Type

struct ComplexCooMatrix { /* private fields */ }

Implementations

source

impl ComplexCooMatrix

source

pub fn assign_real( &mut self, alpha: f64, beta: f64, other: &CooMatrix ) -> Result<(), StrError>

Assigns this matrix to the values of another real matrix (scaled)

Performs:

this = (α + βi) · other

Thus:

this[p].real = α · other[p]
this[p].imag = β · other[p]

other[p] ∈ Reals
p = [0, nnz(other)]

Warning: make sure to allocate max_nnz ≥ nnz(other).

source

pub fn augment_real( &mut self, alpha: f64, beta: f64, other: &CooMatrix ) -> Result<(), StrError>

Augments this matrix with the entries of another real matrix (scaled)

Effectively, performs:

this += (α + βi) · other

Thus:

this[p].real += α · other[p]
this[p].imag += β · other[p]

other[p] ∈ Reals
p = [0, nnz(other)]

Warning: make sure to allocate max_nnz ≥ nnz(this) + nnz(other).